// Economy.java                                                -*- Java -*-
//     The economy of the entire system
//
// COPYRIGHT (C) 1998, Bradley M. Kuhn
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//
// Written   :   Bradley M. Kuhn         University of Cincinnati
//   By          
//
// Written   :   John Franco
//   For         Special Topics: Java Programming
//               15-625-595-001, Fall 1998
// RCS       :
//
// $Source: /home/server/Project/RCS/Economy.java $
// $Revision: 0.11 $
// $Date: 1998/12/15 05:56:01 $
//
// $Log: Economy.java $
// Revision 0.11  1998/12/15 05:56:01  bkuhn
//   -- put files under the GPL
//
// Revision 0.10  1998/12/02 07:37:52  bkuhn
//   -- commenting changes
//
// Revision 0.8  1998/11/30 09:13:17  bkuhn
//   -- added holders to make sure we always have enough folk holding
//      a given resource
//   -- moved stuff that updatedStatistics() was printing to printStatistics()
//
// Revision 0.7  1998/11/29 23:24:21  bkuhn
//   -- decreased Economy max out value from 1000000 to 10000
//
// Revision 0.6  1998/11/29 12:01:49  bkuhn
//   # changed a few things from int to long
//   -- changed updateStatistics so it now writes a file and has
//      a reasonable algorithm
//   -- write trade() so that Players can now trade with the Monitor
//
// Revision 0.5  1998/11/25 04:36:01  bkuhn
//   -- made it so that someone is never given more than half of thier
//      allotment to a single resource
//
// Revision 0.4  1998/11/18 07:51:45  bkuhn
//   # moved ALLOTMENT_AMOUNT_TO_GIVE constant to GameParameters
//
// Revision 0.3  1998/11/16 07:47:24  bkuhn
//   -- got getNewDistributionVector() working
//   -- fixed typo in hash table building in constructor
//
// Revision 0.2  1998/11/15 17:12:02  bkuhn
//   # began working on minor changes that aren't doing much yet
//
// Revision 0.1  1998/11/13 10:57:19  bkuhn
//   # initial version
//

import java.util.*;
import java.security.*;
import java.io.*;

/*****************************************************************************/
class Economy implements Serializable
{
      Hashtable market;
      Hashtable holders;

      static SecureRandom random = new SecureRandom();

      /*********************************************************************/
      Economy()
      {
         market = new Hashtable(10);
         holders = new Hashtable(10);

          market.put("rupyulars", new ResourceValue("rupyulars"));

          market.put("oil", new ResourceValue("oil"));
          market.put("steel", new ResourceValue("steel"));
          market.put("plastic", new ResourceValue("plastic"));
          market.put("copper", new ResourceValue("copper"));
          market.put("glass", new ResourceValue("glass"));
          market.put("rubber", new ResourceValue("rubber"));

         for (Enumeration e = market.keys() ; e.hasMoreElements() ;)
            holders.put(e.nextElement(), new Integer(0));

          market.put("weapons", new ResourceValue("weapons"));
          market.put("computers", new ResourceValue("computers"));
          market.put("vehicles", new ResourceValue("vechicles"));
      }
      /*********************************************************************/
      public Enumeration getResourceNames()
      {
         return market.keys();
      }
      /*********************************************************************/
      public ResourceValue getResourceValueByName(String name)
      {
         return (ResourceValue) market.get(name);
      }
      /*********************************************************************/
      void increaseHolder(String key, int amount)
      {
         holders.put(key, new Integer( ((Integer)
                                       holders.get(key)).intValue() + 
                                       amount));
      }
      /*********************************************************************/
      boolean reasonableHoldings()
      {
         for (Enumeration e = holders.keys() ; e.hasMoreElements() ;)
         {
            String currentHolding = (String) e.nextElement();

            if ( ((Integer) holders.get(currentHolding)).intValue() < 
               GameParameters.MINIMUM_PLAYERS_THAT_MUST_HOLD_EACH_RESOURCE)
               return false;
         }
         return true;
      }
      /*********************************************************************/
      public Hashtable getNewDistributionHash()
      {
         Hashtable distribution = new Hashtable(holders.size());

         String[] names =  new String[holders.size()];

         Enumeration e = holders.keys();

         String firstOne = null;

         for (int ii = 0;
                 e.hasMoreElements() ; ii++)
         {
            names[ii] = (String) e.nextElement();
            distribution.put(names[ii], new Long(0));

            int holderCount =  ( (Integer) holders.get(names[ii])).intValue();

            if (holderCount < 
                GameParameters.MINIMUM_PLAYERS_THAT_MUST_HOLD_EACH_RESOURCE)
            {
               if (firstOne == null)
                  firstOne = names[ii];
               else 
               {
                  if (random.nextDouble() <= 0.50)
                     firstOne = names[ii];
               }
            }
         }
         long remainingAllotment = GameParameters.ALLOTMENT_AMOUNT_TO_GIVE;

         while (remainingAllotment > 0)
         {
            String currentHolding;
            do
            {
               if (firstOne != null)
               {
                  currentHolding = firstOne;
                  firstOne = null;
               }
               else
               {
                  int ii = random.nextInt();
                  if (ii < 0)
                     ii = - ii;
                  
                  ii %= names.length;
                  currentHolding = names[ii];
               }
            } while(
               ( (Long) distribution.get(currentHolding)).longValue()
               >= GameParameters.ALLOTMENT_AMOUNT_TO_GIVE / 2);

            // Now, pick a random amount to give them.

            long oldValue  = ((Long) distribution.get(currentHolding))
                                .longValue();
            long amount;
            do
            {
               amount = random.nextLong();
               if (amount < 0) amount = - amount;

               amount %= remainingAllotment + 1;

            } while (amount + oldValue
                        >= GameParameters.ALLOTMENT_AMOUNT_TO_GIVE / 2);
                             // never give more than half total in one thing
            remainingAllotment -= amount;

            distribution.put(currentHolding, new Long(amount + oldValue));
         }            
         return distribution;
      }
      /**********************************************************************/
      private double getRandomDoubleBetween(double min, double max)
      {
         double rand = random.nextDouble();
         if (rand < 0) rand = - rand;
         return ( (rand * (max - min) ) + min );
      }
      /*********************************************************************/
      public synchronized void updateStatistics()
      {
         long rupyulars =  
            ((ResourceValue) market.get("rupyulars")).getAmount();

         if (rupyulars == 0)
            rupyulars = 1;

         for (Enumeration e = market.keys() ; e.hasMoreElements() ;)
         {
            String key = (String) e.nextElement();

            if (key.equalsIgnoreCase("rupyulars"))
               continue;

            ResourceValue rv = (ResourceValue) market.get(key);

            long amount = rv.getAmount();

            double newValue =  10000.0;

            if (amount > 0)
            {
               double change = ( ( (double) rupyulars) /  amount)
                                        - rv.getMarketValue();

               double delta = getRandomDoubleBetween(0.1 * change, change);

               newValue = rv.getMarketValue() + delta;
            }
            rv.setMarketValue(newValue);
         }
      }
      /*********************************************************************/
      public synchronized void printStatistics(PrintStream file)
      {
         if (file == null)
            return;

         file.println("ECONOMY STATISTICS");

         long rupyulars =  
            ((ResourceValue) market.get("rupyulars")).getAmount();

         file.println("     rupyulars: " + " count = " + rupyulars +
                      ", value = 1.00000");

         for (Enumeration e = market.keys() ; e.hasMoreElements() ;)
         {
            String key = (String) e.nextElement();

            if (key.equalsIgnoreCase("rupyulars"))
               continue;

            ResourceValue rv = (ResourceValue) market.get(key);

            file.println("     " + key + ": " + " count = " + rv.getAmount() +
                         ", value = " + rv.getMarketValue() +
                         ", monitorHolds = " + rv.getMonitorAmount());
         }
      }         
      /*********************************************************************/
      // This methods decreases 
      // names of resources
      synchronized boolean trade(Player player, 
                                 String resourceFrom, long amountFrom,
                                 String resourceTo, long amountTo)
         throws UnknownResourceException
      {
         resourceTo = resourceTo.toLowerCase();
         resourceFrom = resourceFrom.toLowerCase();

         ResourceValue rvDesired = (ResourceValue) market.get(resourceTo);

         if (rvDesired == null)
            throw new UnknownResourceException(resourceTo +
                                               " is not a known resource");

         if (rvDesired.getMonitorAmount() < amountTo)
            return false;

         double valueOfDesired = amountTo * rvDesired.getMarketValue();
         valueOfDesired += GameParameters.MONITOR_MARKUP_PERCENT *
                                valueOfDesired;

         ResourceValue rvGiven = (ResourceValue) market.get(resourceFrom);

         if (rvGiven == null)
            throw new UnknownResourceException(resourceFrom +
                                               " is not a known resource");

         double valueOfGiven = amountFrom * rvGiven.getMarketValue();

         if (valueOfDesired > valueOfGiven)
            return false;
         else
         {
            try
            {
               player.getWealth().changeHolding(resourceFrom, 0 - amountFrom);
               rvGiven.changeMonitorAmount(amountFrom);

               player.getWealth().changeHolding(resourceTo, amountTo);
               rvDesired.changeMonitorAmount(0 - amountTo);
            }
            catch (InsufficientResourceException ire)
            {
               System.out.println("FATAL MONTIOR TRADE ERROR:  (" + 
                                  ire.getMessage() + ")");
            }

            return true;
         }
      }         
}
/*
  Local Variables:
  tab-width: 4
  indent-tabs-mode: nil
  eval: (c-set-style "ellemtel")
  End:
*/
