// MyPlayer.java                                              -*- Java -*-
//    An implementation of a reasonable player for the game
//
// COPYRIGHT (C) 1998 Bradley M. Kuhn, All Rights Reserved.
//
// Written   :   Bradley M. Kuhn         University of Cincinnati
//   By          
//
// Written   :   John Franco
//   For         Special Topics: Java Programming
//               15-625-595-001, Fall 1998
// RCS       :
//
// $Source: /home/server/MyPlayer/RCS/MyPlayer.java $
// $Revision: 0.2 $
// $Date: 1998/11/09 07:25:27 $
//
// $Log: MyPlayer.java $

import java.io.*;
import java.math.*;
import java.security.*;
import java.net.*;
import java.util.*;
/*****************************************************************************/
class ResultHandler
{
      static String HandleIt(String directive[], 
                             ConnectionHandler connectionHandler)
      {
         String result = "";

         if (directive[1].equalsIgnoreCase("PASSWORD"))
         {
            connectionHandler.setMonitorPassword(directive[2]);
            result = ResultHandler.buildStringFromArray(directive, 1);
         }
         if (directive[1].equalsIgnoreCase("CHANGE_PASSWORD"))
         {
            connectionHandler.getPlayer().changePassword(directive[2]);
            result = ResultHandler.buildStringFromArray(directive, 1);
         }
         else if (directive[1].equalsIgnoreCase("PLAYER_STATUS"))
         {
            result = ResultHandler.buildStringFromArray(directive, 2);
            connectionHandler.getPlayer().setWealth(result);
         }
         else if (directive[1].equalsIgnoreCase("GET_GAME_IDENTS"))
         {
            result = ResultHandler.buildStringFromArray(directive, 2);
            connectionHandler.getPlayer().updateEnemies(result);
         }
         else if (directive[1].equalsIgnoreCase("RANDOM_PLAYER_HOST_PORT"))
         {
            result = ResultHandler.buildStringFromArray(directive, 2);
            connectionHandler.getPlayer().handleEnemyHostPort(result);
         }
         else if (directive[1].equalsIgnoreCase("IDENT"))
         {
            connectionHandler.setDHExchangeKey(directive[2]);
            result = ResultHandler.buildStringFromArray(directive, 1);
         }
         else
         {
            result = ResultHandler.buildStringFromArray(directive, 1);
         }
         return result;
      }
      /*********************************************************************/
      public static String buildStringFromArray(String a[], int start)
      {
         String answer = "";
         for(int ii = start; ii < a.length; ii++)
            answer = new String(answer + a[ii] + " ");

         return answer;
      }
}
/*****************************************************************************/
class MessageGroupHandler
{
      String requiredCommand;
      String result;
      String comment;
      String commandError;
      
      String tradeData;

      ConnectionHandler connectionHandler;
      /*********************************************************************/
      MessageGroupHandler(ConnectionHandler conHandler)
         throws IOException
      {
         requiredCommand = null;
         result = null;
         comment = null;
         commandError = null;
      
         connectionHandler = conHandler;
         parseInput(conHandler.who());
      }
      /*********************************************************************/
      public String getResult()
      {
         return result;
      }
      /*********************************************************************/
      public String getCommandError()
      {
         return commandError;
      }
      /*********************************************************************/
      private void parseInput(String who)
         throws IOException
      {
         MyPlayer player = connectionHandler.getPlayer();
         
         boolean inMessageGroup = true;

         tradeData = null;

         while (inMessageGroup)
         {
            String directiveLine = connectionHandler.readLine();

            if (directiveLine == null)
               throw new IOException("input exahusted");

            Vector argv = new Vector();
            
            StringTokenizer st = new StringTokenizer(directiveLine);
            
            while (st.hasMoreTokens())
               argv.addElement(st.nextToken().toUpperCase());
            
            String directive[] = new String[argv.size()];
            argv.copyInto(directive);

            if (directive[0].equalsIgnoreCase("REQUIRE:"))
            {
               requiredCommand = directive[1];
            }
            else if (directive[0].equalsIgnoreCase("COMMENT:"))
            {
               comment = ResultHandler.buildStringFromArray(directive, 1);
            }
            else if (directive[0].equalsIgnoreCase("COMMAND_ERROR:"))
            {
               commandError = ResultHandler.buildStringFromArray(directive, 1);
            }
            else if (directive[0].equalsIgnoreCase("RESULT:"))
            {
               result = ResultHandler.HandleIt(directive, connectionHandler);
            }
            else if (directive[0].equalsIgnoreCase(
                                               "PLAYER_PASSWORD_CHECKSUM:"))
            {
               connectionHandler.setPlayerPasswordChecksum(directive[1]);
            } 
            else if (directive[0].equalsIgnoreCase("TRADE:"))
            {
               tradeData =  ResultHandler.buildStringFromArray(directive, 1);
            }
            else if (directive[0].equalsIgnoreCase("WAITING:"))
            {
               inMessageGroup = false;
            } 
         }
         if (comment != null)
            System.err.println(who + " COMMENT RECEIVED: " + comment);
         if (result != null)
            System.err.println(who + " RESULT RECEIVED: " + result);
         if (commandError != null)
            System.err.println(who + " COMMAND ERROR RECEIVED: " + 
                               commandError);
         
      }            
      /*********************************************************************/
      public boolean somethingRequired()
      {
         return (requiredCommand != null);
      }
      /*********************************************************************/
      protected String fillInArguments()
      {
         MyPlayer player = connectionHandler.getPlayer();

         String args = new String(requiredCommand + " ");

         if (requiredCommand.equalsIgnoreCase("IDENT"))
         {
            args = new String(args +
                              player.getIdentity()
                              + " " + connectionHandler.getDHExchangeKey());
         }
         else if (requiredCommand.equalsIgnoreCase("PASSWORD"))
         {
            args = new String(args + player.getPlayerPassword());
         }
         else if (requiredCommand.equalsIgnoreCase("ALIVE"))
         {
            args = new String(args + player.getMonitorPassword());
         }
         else if (requiredCommand.equalsIgnoreCase("HOST_PORT"))
         {
            args = new String(args + player.getHost() + " " + 
                              player.getPort());
         }
          else if (requiredCommand.equalsIgnoreCase("TRADE_RESPONSE"))
          {
             if (player.acceptTrade(tradeData))
             {
                args = new String(args + "ACCEPT");
                System.out.println("TRADE: ACCEPTED: " + tradeData);
             }                                   
             else
             {
                args = new String(args + "DECLINE");
                System.out.println("TRADE: DECLINED: " + tradeData);
             }
          }
         else if (requiredCommand.equalsIgnoreCase("WAR_TRUCE_RESPONSE"))
         {
            args = new String(args + "ACCEPT");
         }
         else if (requiredCommand.equalsIgnoreCase("WAR_DEFEND"))
         {
            int weapons = player.getWealth().getHoldingByName("weapons");
            int vehicles =  player.getWealth().getHoldingByName("vehicles");
            if (weapons > vehicles)
               weapons = vehicles;
            else 
               vehicles = weapons;

            vehicles /= 2;    weapons /= 2;

            player.getWealth().increaseHoldingByName("weapons", 0 - weapons);
            player.getWealth().increaseHoldingByName("vehicles", 0- vehicles);
            
            args = new String(args + " " + weapons + " " + vehicles);
         }
         return args;
      }
      /*********************************************************************/
      protected void sendCommand(String command, PrintStream output)
      {
         System.err.println(connectionHandler.who() +
                            " SENDING COMMAND: \"" + command + "\"");
         connectionHandler.println(command);
      }
      /*********************************************************************/
      public boolean grantRequest(PrintStream output)
      {
         boolean returnValue = false;

         if (somethingRequired())
         {
            String args = fillInArguments();
            
            if (requiredCommand.equalsIgnoreCase("IDENT"))
            {
                  sendCommand(args, output);
                  returnValue = true;
            }
         }
         return returnValue;
      }
}
/****************************************************************************/
class PlayerMessageGroupHandler extends MessageGroupHandler
{
      /*********************************************************************/
      PlayerMessageGroupHandler(ConnectionHandler conHandler)
         throws IOException
      {
         super(conHandler);
      }
}
/****************************************************************************/
class MonitorMessageGroupHandler extends MessageGroupHandler
{
      /*********************************************************************/
      MonitorMessageGroupHandler(ConnectionHandler conHandler)
         throws IOException
      {
         super(conHandler);
      }
      /*********************************************************************/
      public boolean grantRequest(PrintStream output)
      {
         boolean returnValue = false;

         if (connectionHandler.isMonitor() &&
             somethingRequired())
         {
            sendCommand(fillInArguments(), output);
            returnValue = true;
         }
         return returnValue;
      }
}
/*****************************************************************************/
class ConnectionHandler extends Thread
{
      Socket connection;
      BufferedReader  input;
      PrintStream     output;
      String monitorPassword = null;
      String playerPasswordChecksum = null;
      String connectingPlayerID = null;
      Cipher cipher;
      DHEngine dhEngine;
      MyPlayer player;

      /*********************************************************************/
      ConnectionHandler(Socket connection, MyPlayer p)
      {
         monitorPassword = "";
         player = p;

         try
         {
            input  = new BufferedReader(new InputStreamReader(
               connection.getInputStream())); 
            
            output = new PrintStream(connection.getOutputStream());
            //             output = new PrintWriter(incoming.getOutputStream());
         } 
         catch (IOException ie)
         {
            System.out.println(
               "ERROR: Caught an I/O exception when creating server");
         }
         cipher = null;
         dhEngine = null;
      }
      /*********************************************************************/
      MyPlayer getPlayer()
      {
         return player;
      }
      /*********************************************************************/
      String getDHExchangeKey()
      {
         dhEngine = new DHEngine(player.getDhKey());
         return dhEngine.getExchangeKey();
      }
      /*********************************************************************/
      void println(String s)
      {
         if (cipher == null)
            output.println(s);
         else
         {
            s = cipher.Encrypt(s);
            output.println(s);
         }
      }
      /*********************************************************************/
      String readLine()
         throws IOException
      {
         if (cipher == null)
            return input.readLine();
         else
         {
            String s = input.readLine();

            if (s != null)
                s = cipher.Decrypt(s);
            return s;
         }
      }
      /*********************************************************************/
      void setDHExchangeKey(String key)
      {
         dhEngine.setExchangeKey(key);
         cipher = new Cipher(dhEngine.getSharedKey());
      }
      /*********************************************************************/
      Cipher getCipher()
      {
         return cipher;
      }
      /*********************************************************************/
      void setCipher(Cipher c)
      {
         cipher = c;
      }
      /*********************************************************************/
      public String who()
      {
         return "GENERIC";
      }
      /*********************************************************************/
      public BufferedReader getInputStream()
      {
         return input;
      }
      /*********************************************************************/
      public void setPlayerPasswordChecksum(String newChecksum)
      {
         playerPasswordChecksum = new String(newChecksum);
      }
      /*********************************************************************/
      public void setMonitorPassword(String newPassword)
      {
         monitorPassword = new String(newPassword);
         player.setMonitorPassword(monitorPassword);
      }
      /*********************************************************************/
      public boolean isMonitor()
      {
         return
            (monitorPassword != null &&
             player.getMonitorPassword() != null &&
             monitorPassword.equalsIgnoreCase(player.getMonitorPassword()));
      }
      /*********************************************************************/
      public boolean isPlayer()
      {
         return (connectingPlayerID != null);
      }
      /*********************************************************************/
      protected MessageGroupHandler getMessageGroupHandler()
         throws IOException
      {
         if (isMonitor())
            return new MonitorMessageGroupHandler(this);
         else if (isPlayer())
            return new PlayerMessageGroupHandler(this);
         else
            return new MessageGroupHandler(this);
      }
}
/*****************************************************************************/
class ServerConnectionHandler extends ConnectionHandler
{
      /*********************************************************************/
      public String who()
      {
         return "SERVER";
      }
      /*********************************************************************/
      public boolean isMonitor()
      {
         return
            ( playerPasswordChecksum != null && 
              playerPasswordChecksum.equalsIgnoreCase(
                 player.getPlayerPasswordChecksum()) ) ||
            super.isMonitor();
      }
      /*********************************************************************/
      ServerConnectionHandler(Socket connection, MyPlayer p)
      {
         super(connection, p);
      }
      /*********************************************************************/
      public void run()
      {
         try
         {
            while (input != null)
            {
               MessageGroupHandler messageGroupHandler
                  = getMessageGroupHandler();

               boolean giveUp = false;

               if (messageGroupHandler.somethingRequired())
               {
                  if (! messageGroupHandler.grantRequest(output) )
                     giveUp = true;
               }
               else {
                  messageGroupHandler.sendCommand("QUIT", output);
                  giveUp = true;
               }
               if (giveUp)
               {
                  if (connection != null) connection.close();
                  monitorPassword = null;    connection = null;
                  input = null;              output = null;
               }
               
            }
         }
         catch (IOException ioe) {}
      }
}
/*****************************************************************************/
class ClientConnectionHandler extends ConnectionHandler
{
      String result = null;
      String commandError = null;
      boolean commandsSent = false;

      Vector commandsToSend;
      HostPort hostPort;

      ClientConnectionHandler(Socket connection, MyPlayer p, Vector commands)
      {
         super(connection, p);
         commandsToSend = commands;
         hostPort = new HostPort(connection.getInetAddress().
                                          getHostName().toUpperCase());
      }
      /*********************************************************************/
      ClientConnectionHandler(InetAddress address, int p,  MyPlayer pl, 
                              Vector commands)
         throws IOException
      {
         super(ClientConnectionHandler.buildConnection(address, p), pl);
         commandsToSend = commands;

         hostPort = new HostPort(address.getHostName().toUpperCase(), p);
      }
      /*********************************************************************/
      ClientConnectionHandler(String host, int p, MyPlayer pl, Vector commands)
         throws UnknownHostException, IOException
      {
         super(ClientConnectionHandler.buildConnection(host, p), pl);

         hostPort = new HostPort(host, p);

         commandsToSend = commands;
      }
      /*********************************************************************/
      Vector getRemainingCommands()
      {
         return commandsToSend;
      }
      /*********************************************************************/
      public String who()
      {
         return "CLIENT";
      }
      /**********************************************************************/
      MyPlayer getPlayer()
      {
         return player;
      }
      /**********************************************************************/
      public void setMonitorPassword(String newPassword)
      {
         monitorPassword = new String(newPassword);
      }
      /*********************************************************************/
      static private Socket buildConnection(String host, int p)
         throws UnknownHostException, IOException
      {
         InetAddress addressList[] = InetAddress.getAllByName(host);

         // Just take the first one...have to pick one....
         return ClientConnectionHandler.buildConnection(addressList[0], p);
      }
      /*********************************************************************/
      static private Socket buildConnection(InetAddress addr, int port)
         throws IOException
      {
         return new Socket(addr, port);
      }
      /*********************************************************************/
      public boolean isMonitor()
      {
         return (hostPort.equals(player.MONITOR_SERVER_HOST, 
                                 player.MONITOR_SERVER_PORT) ||
                 super.isMonitor());
      }
      /*********************************************************************/
      protected String sendOneOfMyCommands(MessageGroupHandler
                                         messageGroupHandler)
      {
         String command = null;
         try
         {
            command = (String) commandsToSend.firstElement();
            commandsToSend.removeElementAt(0);

            if (command != null)
               messageGroupHandler.sendCommand(command, output);
         }
         catch (NoSuchElementException nse)
         {
            command = null;
         }
         commandsSent = (command == null);
         return command;
      }
      /*********************************************************************/
      boolean allCommandsHaveBeenSent()
      {
         return commandsSent;
      }
      /*********************************************************************/
      void recoverOutput(MessageGroupHandler messageGroupHandler)
      {
         result       = messageGroupHandler.getResult();
         commandError = messageGroupHandler.getCommandError();
      }
      /*********************************************************************/
      public void run()
      {
         try
         {
            String command = null;
            while (input != null)
            {
               MessageGroupHandler messageGroupHandler
                  = getMessageGroupHandler();

               if (command != null)
               {
                  recoverOutput(messageGroupHandler);
                  if (command.equalsIgnoreCase("PLAYER_STATUS")) 
                     if (result != null)
                        System.out.println("STATUS: " + result);
               }                  
               boolean giveUp = false;

               if (messageGroupHandler.somethingRequired())
               {
                  if (! messageGroupHandler.grantRequest(output) )
                     giveUp = true;
               }
               else
               {
                  boolean sendQuit = false;

                  if (! allCommandsHaveBeenSent())
                  {
                     command = sendOneOfMyCommands(messageGroupHandler);
                     if (command == null)
                        sendQuit = true;
                  }
                  else
                     sendQuit = true;
                     
                  if (sendQuit)
                  {
                     command = "QUIT";
                     messageGroupHandler.sendCommand(command, output);
                     
                     giveUp = true;
                  }
               }
               if (giveUp)
               {
                  if (connection != null) connection.close();
                  monitorPassword = null; connection = null;
                  input = null;           output = null;
               }
            }
         }
         catch (IOException ioe) {}

         if (! allCommandsHaveBeenSent())
            System.out.println("ERROR: UNABLE TO SEND COMMANDS!" +
                               commandsToSend);
      }
      /*********************************************************************/
} 
/*****************************************************************************/
class TradeClientConnectionHandler extends ClientConnectionHandler
{
      /*********************************************************************/
      TradeClientConnectionHandler(String host, int p, MyPlayer pl)
         throws UnknownHostException, IOException
      {
         super(host,  p,   pl,  new Vector());
      }
      /*********************************************************************/
      TradeClientConnectionHandler(String host, int p, MyPlayer pl,
                                   Vector v)
         throws UnknownHostException, IOException
      {
         super(host,  p,   pl,  v);
      }
      /*********************************************************************/
      public void run()
      {
         try
         {
            // First set up intitial trade commands

            if (commandsToSend.isEmpty())
               for (Enumeration e = player.getWealth().getNeeded() ;
                    e.hasMoreElements() ; )
               {
                  String neededResource = (String) e.nextElement();
                  
                  int willingToGive = (int) (1 * player.getWealth().
                                             getFactor(neededResource));
                  
                  for (Enumeration f = player.getWealth().
                          getExcessHoldings().elements();
                       f.hasMoreElements() ;)
                  {
                     String giveResource = (String) f.nextElement();

                     if (giveResource.equalsIgnoreCase(neededResource))
                        continue;

                     int askForAmount;
                     int offerAmount =  (willingToGive > player.getWealth().
                                           getHoldingByName(giveResource) -2) ?
                       (player.getWealth().getHoldingByName(giveResource) -2) :
                         willingToGive;

                     if (player.getWealth().isExpensiveResource(giveResource)
                        && (! player.getWealth().
                                        isExpensiveResource(neededResource)) )
                     {
                        offerAmount = 1 + (int) (player.getWealth().
                                           getFactor(neededResource) /
                                           PlayerParameters.
                                                 EXPENSIVE_RESOURCE_OFFSET);
                        askForAmount = (int)
                           (PlayerParameters.EXPENSIVE_RESOURCE_OFFSET /
                                        player.getWealth().
                                             getFactor(neededResource));
                        System.out.println("DEBUG: willing to give expensive "
                                           + giveResource + "(" +
                                           offerAmount + ") for cheap " +
                                           neededResource + "(" +
                                           askForAmount + ")");
                     }
                     else
                        askForAmount = 1;


                     String curCommandStart = "TRADE_REQUEST " +
                        player.getIdentity() + " " + 
                        giveResource + " " + offerAmount +
                        " for ";
                     
                     for (Enumeration g = 
                             player.getEnemyDB().getEnemyIdents(); 
                          g.hasMoreElements() ;)
                     {
                        String newCommand = curCommandStart + 
                           ( (String) g.nextElement()) +
                           " " + neededResource + " " + askForAmount;
                        if (! commandsToSend.contains(newCommand))
                           commandsToSend.addElement(newCommand);
                     }
                     String newCommand = curCommandStart + "MONITOR " +
                        neededResource + " " + askForAmount;
                     if (! commandsToSend.contains(newCommand))
                        commandsToSend.addElement(newCommand);
                  }
               }
            String command = null;

            while (input != null)
            {
               MessageGroupHandler messageGroupHandler
                  = getMessageGroupHandler();

                if (command != null)
               {
                  recoverOutput(messageGroupHandler);
                  if (result != null)
                  {
                     StringTokenizer st = new StringTokenizer(result);
                     
                     boolean accepted = false;

                     while (st.hasMoreTokens())
                        if (st.nextToken().equalsIgnoreCase("ACCEPTED"))
                        {
                           accepted = true;
                           break;
                        }

                     if (accepted)
                     {
                        System.out.println("TRADE:  MY_REQUEST_ACCEPTED: " +
                                           command);
                        st = new StringTokenizer(command);
                        Vector argv = new Vector();

                        while (st.hasMoreTokens())
                           argv.addElement(st.nextToken().toUpperCase());

                        String trade[] = new String[argv.size()];
                        argv.copyInto(trade);

                        int hisValue, myValue;
                        
                        try
                        {
                           myValue = Integer.parseInt(trade[3]);
                           hisValue  = Integer.parseInt(trade[7]);

                           player.getWealth().increaseHoldingByName(trade[2],
                                                                0 - myValue);
                           player.getWealth().increaseHoldingByName(trade[6],
                                                                hisValue);
 
                           System.out.println("DEBUG: " + trade[2] + 
                                              " is now decreased by " + myValue
                                              + " because trade suceeded");
                           System.out.println("DEBUG: " + trade[6] + 
                                              " is now increased by " +
                                              hisValue
                                              + " because trade suceeded");
                       }
                        catch (NumberFormatException nfe)
                        {
                           System.out.println("TRADE: " + nfe + " on " +
                                              trade[3]
                                              + " and/or " + trade[7]);
                        }

                        if (player.getWealth().getHoldingByName(trade[6]) <
                            player.getWealth().getHoldingByName(trade[2]))
                           commandsToSend.addElement(command);
                        // resend the command, see if we can get more, and if 
                        //  we still want it (that's what the if tests)
                     }
                  }
                        
                  if (commandError != null)
                  {
                     commandError.toUpperCase();
                     if (commandError.lastIndexOf("MAKE CONNECTION") == -1)
                        System.out.println("ERROR:  WHEN TRADING: " + 
                                           commandError);
                  }
               }

               boolean giveUp = false;

               if (messageGroupHandler.somethingRequired())
               {
                  if (! messageGroupHandler.grantRequest(output) )
                     giveUp = true;
               }
               else
               {
                  if (! allCommandsHaveBeenSent())
                  {
                     command = sendOneOfMyCommands(messageGroupHandler);
                  }
                  else
                  {
                     messageGroupHandler.sendCommand("QUIT", output);
                     giveUp = true;
                  }
               }
               if (giveUp)
               {
                  if (connection != null) connection.close();
                  monitorPassword = null; connection = null;
                  input = null;           output = null;
               }
            }
         }
         catch (IOException ioe) {}
      }
      
      
}
/*****************************************************************************/
class MyPlayer extends Thread implements Serializable
{
      public static final String MY_PLAYER_FILE = "./bkuhn.MyPlayer";

      public String MONITOR_SERVER_HOST;
      public int MONITOR_SERVER_PORT;
      
      private String PLAYER_SERVER_HOST;

      Password password;
      HostPort hostPort;
      PlayerWealth wealth;
      EnemyDB enemyDB;

      String playerPasswordPending;
      
      /**********************************************************************/
      public MyPlayer(String ident,
                      String monitorServer, String monitorServerPort, 
                      String playerServer,
                      String playPass, String monPass)
         throws NumberFormatException
      {
         playerPasswordPending = null;

         if (playerServer != null)
         {
            PLAYER_SERVER_HOST = playerServer;
            hostPort = new HostPort(playerServer);
         }
         else
            hostPort = new HostPort(PLAYER_SERVER_HOST);


         MONITOR_SERVER_HOST = monitorServer;

         int monPort = Integer.parseInt(monitorServerPort);

         if (monPort > 0)
             MONITOR_SERVER_PORT = monPort;

         if (playPass == null)
         {
            try
            {
               FileInputStream fis = new FileInputStream(Password.FILE);
               ObjectInputStream ois = new ObjectInputStream(fis);
               password = (Password) ois.readObject();
               ois.close();
            }
            catch (Exception e)
            {
               System.out.println("ERROR: UNABLE TO GET Password object " + 
                                  e);
            }
         }
         else 
         {
            password = new Password(ident, playPass, monPass);
         }

         enemyDB = new EnemyDB();
      }
      /***********************************************************************/
      static public MyPlayer readMyPlayer(String filename)
      {
         MyPlayer player = null;
         try
         {
            FileInputStream fis = new FileInputStream(filename);
            ObjectInputStream ois = new ObjectInputStream(fis);
            player = (MyPlayer) ois.readObject();
            ois.close();
         }
         catch (Exception e)
         {
            System.out.println("ERROR: UNABLE TO GET MyPlauer object " + 
                               e);
            player = null;
         }
         return player;
      }
      /***********************************************************************/
      synchronized public void setPlayerPasswordPending(String s)
      {
         playerPasswordPending = s;
      }
      /***********************************************************************/
      synchronized public void clearPlayerPasswordPending()
      {
         playerPasswordPending = null;
      }
      /***********************************************************************/
      synchronized void changePassword(String monPass)
      {
         if (playerPasswordPending == null)
         {
            System.out.println("ERROR: FATAL: pending player password missing "
                               + " was player=" + password.getPlayerPassword()
                               + "  mon=" + password.getMonitorPassword());
            System.exit(0);
         }

         password.changePassword(playerPasswordPending, monPass);

         playerPasswordPending = null;
      }         
      /***********************************************************************/
      synchronized public void writeMyPlayer()
      {
         try
         {
            FileOutputStream fos = new FileOutputStream(
                                                 MyPlayer.MY_PLAYER_FILE);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeObject( (Object) this);
            oos.close();
         }
         catch (Exception e)
         {
            System.out.println(
               "ERROR_FATAL: -- cannot serialize MyPlayer: " + e);
         }
      }
      /**********************************************************************/
      void setWealth(String s)
      {
         if (wealth == null)
            wealth = new PlayerWealth(s);
         else
            wealth.updateWealth(s);
      }
      /**********************************************************************/
      EnemyDB getEnemyDB()
      {
         return enemyDB;
      }
      /**********************************************************************/
      void handleEnemyHostPort(String s)
      {
         StringTokenizer st = new StringTokenizer(s);
            
         Vector argv = new Vector();

         while (st.hasMoreTokens())
            argv.addElement(st.nextToken().toUpperCase());

            String data[] = new String[argv.size()];
            argv.copyInto(data);

            int port = -1;
            try
            {
               port = Integer.parseInt(data[2]);
            }
            catch (NumberFormatException nfe) {}
            enemyDB.setAnEnemyHostPort(data[0], data[1], port);
      }
      /**********************************************************************/
      void updateEnemies(String s)
      {
         enemyDB.updateEnemies(this, s);
      }
      /**********************************************************************/
      String getHost()
      {
         return hostPort.getHost();
      }
      /**********************************************************************/
      PlayerWealth getWealth()
      {
         return wealth;
      }
      /**********************************************************************/
      int getPort()
      {
         return hostPort.getPort();
      }
      /**********************************************************************/
      DHKeyObject getDhKey()
      {
         return password.getDhKey();
      }
      /*********************************************************************/
      String getIdentity()
      {
         return password.getIdentity();
      }
      /*********************************************************************/
      String getMonitorPassword()
      {
         return password.getMonitorPassword();
      }
      /*********************************************************************/
      void setMonitorPassword(String s)
      {
         password.setMonitorPassword(s);
      }
      /*********************************************************************/
      String getPlayerPassword()
      {
         return password.getPlayerPassword();
      }
      /*********************************************************************/
      String getPlayerPasswordChecksum()
      {
         return password.getPlayerPasswordChecksum();
      }
      /*********************************************************************/
      boolean acceptTrade(String tradeData)
      {
         if (tradeData == null)
            return false;

         StringTokenizer st = new StringTokenizer(tradeData);
            
         Vector argv = new Vector();

         while (st.hasMoreTokens())
            argv.addElement(st.nextToken().toUpperCase());

            String trade[] = new String[argv.size()];
            argv.copyInto(trade);

            if ( (! trade[4].equalsIgnoreCase(password.getIdentity())) ||
                 (! trade[3].equalsIgnoreCase("for")) )
            {
               System.out.println("ERROR_TRADE: problem with my ident " +
                                  trade[4] + " or for=" + trade[3]);

               return false;
            }
            int hisValue, myValue;

            try
            {
               hisValue = Integer.parseInt(trade[2]);
               myValue  = Integer.parseInt(trade[6]);
            }
            catch (NumberFormatException nfe)
            {
               System.out.println("TRADE: " + nfe + " on " + trade[2]
                                  + " and/or " + trade[6]);
               return false;
            }
            boolean doTrade =  wealth.tradeAmountsFair(trade[5], myValue, 
                                                       trade[1], hisValue);
            if (doTrade)
            {
               wealth.increaseHoldingByName(trade[5], 0 - myValue);
               wealth.increaseHoldingByName(trade[1], hisValue);

               System.out.println("DEBUG: " + trade[5] + 
                                  " is now decreased by " + myValue
                                  + " because trade suceeded");
               System.out.println("DEBUG: " + trade[1] + 
                                  " is now incrased by " + hisValue
                                  + " because trade suceeded");
            }
            return doTrade;
      }
      /**********************************************************************/
      public void run()
      {
         ServerSocket serverSocket = null;
          try 
          {
            serverSocket = new ServerSocket(0);
            System.out.println("PLAYER: listening on port " + 
                               serverSocket.getLocalPort());
          }
          catch (IOException ioe)
          {
             System.out.println("ERROR: Unable to bind to port: " +
                                ioe);
             return;
          }
          hostPort.setPort(serverSocket.getLocalPort());

         try
         {
             new RecurringEvent(Class.forName("ChangePassword"), this, 
                                5.0 * 60.0
                ).start();
             new RecurringEvent(Class.forName("UpdateWorld"), this, 
                                15.0 * 60.0
                ).start();
            new RecurringEvent(Class.forName("SerializeMyPlayer"), this, 
                               6.0 * 60.0
               ).start();
             new RecurringEvent(Class.forName("GetNeededResources"), this, 
                                60.0 * 60.0
                ).start();

            GetThingsStarted gts = new GetThingsStarted(this);
            gts.start();
         }
         catch (ClassNotFoundException cnfe)
         {
            System.out.println(
               "ERROR: Unable to find one of the EVENT classes; " +
               "Problems will occur!");
         }
         try
         {
            while (true)
            {
               Socket incoming = serverSocket.accept();
      
               new ServerConnectionHandler(incoming, this).start();
            }
         }
         catch (IOException ioe)
         {
            System.out.println("ERROR: IOException caught when serving client" +
                                ioe);
         }
      }
      /**********************************************************************/
      public static void main(String arg[])
                          throws NumberFormatException
      {
          MyPlayer server;

          server = MyPlayer.readMyPlayer(MyPlayer.MY_PLAYER_FILE);

          if (server == null)
          {
             if (arg.length == 6)
                server = new MyPlayer(arg[0], arg[1], arg[2], arg[3],
                                      arg[4], arg[5]);
             else if (arg.length == 5)
                server = new MyPlayer(arg[0], arg[1], arg[2], arg[3],
                                      arg[4], null);
             else
                server = new MyPlayer(arg[0], arg[1], arg[2], arg[3],
                                      null, null);
          }
          server.start();
      }
}
/*****************************************************************************/
class ChangePassword extends Thread implements RecurredEvent
{
      static SecureRandom random = new SecureRandom();

      MyPlayer player;

      ChangePassword()
      {
      }
      /**********************************************************************/
      public void initialize(Object o)
      {
         player = (MyPlayer) o;
      }
      /**********************************************************************/
      public void run()
      {
          setPriority(MAX_PRIORITY - 1);

          Date now = new Date();
          
         System.out.println("CHANGE_PASSWORD: Beginning at " + now + "...");

         String newPass = new BigInteger(150, random)
                                           .toString(36).toUpperCase();

         Vector commands = new Vector();
         ClientConnectionHandler connection = null;

         player.setPlayerPasswordPending(newPass);

         commands.addElement("CHANGE_PASSWORD " +  player.getPlayerPassword() +
                             " " + newPass);
         try
         {
            connection =
               new ClientConnectionHandler(player.MONITOR_SERVER_HOST,
                                           player.MONITOR_SERVER_PORT, 
                                           player, commands);
            connection.start();

            try { connection.join(); }
            catch(InterruptedException ie) { }
         }
         catch (UnknownHostException ioe)
         {
            System.out.println("ERROR: Unable to resolve " +
                               player.MONITOR_SERVER_HOST + " " + ioe);
            return;
         }
         catch (IOException ie)
         {
            System.out.println("ERROR: IO Exception got with " +
                               player.MONITOR_SERVER_HOST + " " + ie);
            return;
         }

         if (connection != null && connection.isAlive())
            connection.stop();

         if (! connection.allCommandsHaveBeenSent())
            player.clearPlayerPasswordPending();
            
         now = new Date();

         player.writeMyPlayer();

         System.out.println("CHANGE_PASSWORD: Ending at " + now + "...");
            
      }
      /**********************************************************************/
}
/*****************************************************************************/
class GetThingsStarted extends Thread  {
      MyPlayer player;

      /**********************************************************************/
      GetThingsStarted(MyPlayer p)
      {
         player = p;
      }
      /**********************************************************************/
      public void run() 
      {
            UpdateWorld uw = new UpdateWorld();
            uw.initialize(player);
            uw.start();

            try { uw.join(); }
            catch(InterruptedException ie) { }
         
            GetNeededResources gnr = new GetNeededResources();
            gnr.initialize(player);
            gnr.start();
      }
      /**********************************************************************/
}
/*****************************************************************************/
class UpdateWorld extends Thread implements RecurredEvent {
      MyPlayer player;

      /**********************************************************************/
      UpdateWorld() {}
      /**********************************************************************/
      public void initialize(Object o)
      {
         player = (MyPlayer) o;
      }
      /**********************************************************************/
      public void run()
      {
          Date now = new Date();

         System.out.println("UPDATE_WORLD: Beginning at " + now + "...");

         Vector commands = new Vector();
         ClientConnectionHandler connection = null;

         commands.addElement("GET_GAME_IDENTS");
         commands.addElement("RANDOM_PLAYER_HOST_PORT");
         commands.addElement("PLAYER_STATUS");
         try
         {
            connection =
               new ClientConnectionHandler(player.MONITOR_SERVER_HOST,
                                           player.MONITOR_SERVER_PORT, 
                                           player, commands);
            connection.start();

            try { connection.join(); }
            catch(InterruptedException ie) { }
            
            if (player.getWealth() != null && 
                player.getWealth().canSynthesize() )

            {
               Vector need = player.getWealth().getShouldBuiltVector();

               commands = new Vector();

               if (need.size() > 0)
               {
                  for(int i = 0; i < PlayerParameters.EXPENSIVE_DELTA; i++)
                     for (Enumeration e = need.elements() ;
                          e.hasMoreElements() ; )
                        commands.addElement("SYNTHESIZE " + ( (String)
                                                            e.nextElement()));
                  commands.addElement("PLAYER_STATUS");
               }

               connection = null;
               connection = new ClientConnectionHandler(
                  player.MONITOR_SERVER_HOST,
                  player.MONITOR_SERVER_PORT, 
                  player, commands);
               connection.start();
               
               try { connection.join(); }
               catch(InterruptedException ie) { }
            }
         }            
         catch (UnknownHostException ioe)
         {
            System.out.println("       Unable to resolve " +
                               player.MONITOR_SERVER_HOST + " " + ioe);
            return;
         }
         catch (IOException ie)
         {
            System.out.println("       IO Exception got with " +
                               player.MONITOR_SERVER_HOST + " " + ie);
            return;
         }

         if (connection != null && connection.isAlive())
            connection.stop();

         now = new Date();

         System.out.println("UPDATE_WORLD: Ending at " + now + "...");
      }
      /**********************************************************************/
}
/*****************************************************************************/
class GetNeededResources extends Thread implements RecurredEvent {
      MyPlayer player;

      /**********************************************************************/
      GetNeededResources() {}
      /**********************************************************************/
      public void initialize(Object o)
      {
         player = (MyPlayer) o;
      }
      /**********************************************************************/
      public void run()
      {
         Date now = new Date();
         System.out.println("GET_NEEDED_RESOURCES: Beginning at " + now +
                            "...");

         ClientConnectionHandler connection = null;
         try
         {
            Vector commands = null;
            do 
            {
               if (commands == null)
                  connection =
                     new TradeClientConnectionHandler(
                                                   player.MONITOR_SERVER_HOST,
                                                   player.MONITOR_SERVER_PORT, 
                                                   player);
               else
                  connection =
                     new TradeClientConnectionHandler(
                                                   player.MONITOR_SERVER_HOST,
                                                   player.MONITOR_SERVER_PORT, 
                                                   player, commands);
               connection.start();
            
               try { connection.join(); }
               catch(InterruptedException ie) { }

               commands = connection.getRemainingCommands();
            }  while (! commands.isEmpty());

            commands = new Vector();
                        
            commands.addElement("PLAYER_STATUS");

            connection = null;
            connection = new ClientConnectionHandler(
                                               player.MONITOR_SERVER_HOST,
                                               player.MONITOR_SERVER_PORT, 
                                               player, commands);

            connection.start();
            try { connection.join(); }
            catch(InterruptedException ie) { }
         }
         catch (UnknownHostException ioe)
         {
            System.out.println("ERROR: Unable to resolve " +
                               player.MONITOR_SERVER_HOST + " " + ioe);
            return;
         }
         catch (IOException ie)
         {
            System.out.println("ERROR: IO Exception got with " +
                               player.MONITOR_SERVER_HOST + " " + ie);
            return;
         }

         if (connection != null && connection.isAlive())
            connection.stop();
         
         now = new Date();

         System.out.println("GET_NEEDED_RESOURCES: Ending at " + now + "...");
      }
      /**********************************************************************/
}
/*****************************************************************************/
class HostPort implements Serializable {
      String host;
      int    port;

      HostPort(String h, int p)
      {
         host = h;
         port = p;
      }
      /**********************************************************************/
      HostPort()
      {
         host = "";
         port = -1;
      }
      /**********************************************************************/
      HostPort(String h)
      {
         host = h;
         port = -1;
      }
      /**********************************************************************/
      boolean equals(String h, int p)
      {
         return (h.equalsIgnoreCase(host) && p == port);
      }
      /**********************************************************************/
      void setPort(int p)
      {
         port = p;
      } 
      /**********************************************************************/
      void setHost(String h)
      {
         host = h;
      } 
      /**********************************************************************/
      String getHost()
      {
         return host;
      }
      /**********************************************************************/
      int getPort()
      {
         return port;
      }
      /**********************************************************************/
}
/*****************************************************************************/
class EnemyDB implements Serializable {
      Hashtable enemies;

      /**********************************************************************/
      EnemyDB()
      {
         enemies = new Hashtable();
      }
      /**********************************************************************/
      synchronized void updateEnemies(MyPlayer player, String s)
      {
         StringTokenizer st = new StringTokenizer(s);

         while (st.hasMoreTokens())
         {
            String ident = st.nextToken().toUpperCase();
            if (enemies.get(ident) == null &&
                (! ident.equalsIgnoreCase(player.getIdentity())) )
               enemies.put(ident, new Enemy(ident));
         }
      }
      /**********************************************************************/
      void setAnEnemyHostPort(String ident, String hostname, int port)
      {
         Enemy enemy = (Enemy) enemies.get(ident);
         if (enemy == null)
            enemies.put(ident, new Enemy(ident, hostname, port));
         else
            enemy.setHostPort(hostname, port);
      }
      /**********************************************************************/
      Enumeration getEnemyIdents()
      {
         return enemies.keys();
      }
      /**********************************************************************/
}
/*****************************************************************************/
class HoldingsParser {
      Hashtable parsed;

      HoldingsParser(String s)
      {
         parsed = parse(s);
      }
      /**********************************************************************/
      public Hashtable getParsedData()
      {
         return parsed;
      }
      /**********************************************************************/
      private Hashtable parse(String s)
      {
         Hashtable holdings = new Hashtable();
         StringTokenizer st = new StringTokenizer(s);
         
         try
         {
            while (st.hasMoreTokens())
               holdings.put(st.nextToken().toLowerCase(),
                            new Integer(st.nextToken()));
         }
         catch(NoSuchElementException nsee)
         {
            System.out.println("Broken Holdings string: " + s + 
                               ",  Exception: " + nsee);
         }
         catch(NumberFormatException nfe)
         {
            System.out.println("Broken number in Holdings string: " + s + 
                               ",  Exception: " + nfe);
         }         
         return holdings;
      }
      /**********************************************************************/
}

/*****************************************************************************/
class Enemy implements Serializable {
      String identity;
      Hashtable holdings;
      HostPort hostPort;

      /**********************************************************************/
      Enemy(String ident)
      {
         identity = ident;
         holdings = null;
         hostPort = new HostPort();
      }
      /**********************************************************************/
      Enemy(String ident, String host, int port)
      {
         identity = ident;
         holdings = null;
         hostPort = new HostPort(host, port);
      }
      /**********************************************************************/
      synchronized void setHostPort(String host, int port)
      {
         hostPort.setHost(host);
         hostPort.setPort(port);
      }
      /**********************************************************************/
      HostPort getHostPort()
      {
         return hostPort;
      }
      /**********************************************************************/
      synchronized void updateWealth(String s)
      {
         holdings = (new HoldingsParser(s)).getParsedData();
      }
      /**********************************************************************/
      int getHoldingByName(String holdingName)
      {
         if (holdings == null)
            return -1;
         else 
         {
            holdingName = holdingName.toLowerCase();
         
            return ( ((Integer) holdings.get(holdingName)).intValue());
         }
      }
      /**********************************************************************/
}
/*****************************************************************************/
class PlayerParameters
{
      public static final int EXPENSIVE_DELTA = 10;
      public static final long MILLISECONDS_TO_GET_DESPERATE = 1000 * 
                                                                 60 * 60 * 9;
                                                     // That's nine hours

      public static final int EXPENSIVE_RESOURCE_OFFSET = 50;

      public static final long HAVE_ENOUGH_THRESHOLD  = 1000 * 60 * 15;
}      
/*****************************************************************************/
class PlayerWealth implements Serializable {
      Hashtable holdings;
      Hashtable willTradeFor;

      Hashtable needed;
      /**********************************************************************/
      PlayerWealth(String s)
      {
         holdings = null;
         needed   = new Hashtable();
         willTradeFor = null;

         updateWealth(s);
      }
      /**********************************************************************/
      private synchronized boolean willTradeThisForThat(String thisOne,
                                                        String thatOne)
      {
         Vector v = (Vector) willTradeFor.get(thisOne);

         if (v != null)
            for (Enumeration e = v.elements() ; e.hasMoreElements() ; )
               if (thatOne.equalsIgnoreCase((String) e.nextElement()))
                  return true;

         return false;
      }
      /**********************************************************************/
      synchronized boolean tradeAmountsFair(String thisOne, int thisValue,
                                            String thatOne, int thatValue)
      {
         thisOne = thisOne.toLowerCase();
         thatOne = thatOne.toLowerCase();

         int amountWillingToGive = (int) ( thatValue * getFactor(thisOne) );

         return (willTradeThisForThat(thisOne, thatOne) &&
                 thisValue <= amountWillingToGive);
      }
      /**********************************************************************/
      double getFactor(String which)
      {
         Date when = (Date) needed.get(which);
         Date now = new Date();

         if (when == null)
            return 1.0;
         else
            return (1.0 + ( (now.getTime()  - when.getTime()) /
                           ((double)
                            PlayerParameters.MILLISECONDS_TO_GET_DESPERATE)) );
      }
      /**********************************************************************/
      synchronized void increaseHoldingByName(String name, int value)
      {
         name = name.toLowerCase();

         int currentValue = ( (Integer) holdings.get(name)).intValue();

         holdings.put(name, new Integer(currentValue + value));
      }
      /**********************************************************************/
      int getHoldingByName(String holdingName)
      {
         holdingName = holdingName.toLowerCase();

         return ( ((Integer) holdings.get(holdingName)).intValue());
      }
      /**********************************************************************/
      public Vector getExcessHoldings()
      {
         Vector v = new Vector();

         for (Enumeration e = holdings.keys() ; e.hasMoreElements() ; )
         {
            String holding = (String) e.nextElement();

            if (getHoldingByName(holding) > 2)
               v.addElement(holding);
         }
         return v;
      }
      /**********************************************************************/
      synchronized boolean canSynthesize()
      {
         int steel = getHoldingByName("steel");
         int copper = getHoldingByName("copper");
         int plastic = getHoldingByName("plastic");
         int glass = getHoldingByName("glass");
         int rubber = getHoldingByName("rubber");
         int oil = getHoldingByName("oil");

         return (   ( copper >= 2 && plastic >= 2 && glass >= 2)  ||
                    ( steel >= 2  && plastic >= 2 && oil >=2 )    ||
                    ( steel >= 2  && rubber >= 2 && oil >=2 && glass >= 2) );
      }
      /**********************************************************************/
      synchronized Vector getShouldBuiltVector()
      {
         Vector v = new Vector();

         int weapons = getHoldingByName("weapons");
         int computers = getHoldingByName("computers");
         int vehicles = getHoldingByName("vehicles");

         int maxFactor = weapons / PlayerParameters.EXPENSIVE_DELTA;

         int ii = computers / PlayerParameters.EXPENSIVE_DELTA;

         if (ii > maxFactor) maxFactor = ii;

         ii = vehicles / PlayerParameters.EXPENSIVE_DELTA;

         if (ii > maxFactor) maxFactor = ii;

         int desiredValue = PlayerParameters.EXPENSIVE_DELTA * maxFactor;

         if (vehicles < desiredValue)
            v.addElement("vehicles");

         if (weapons < desiredValue)
            v.addElement("weapons");

         if (computers < desiredValue)
            v.addElement("computers");

         if (v.isEmpty())
         {
            v.addElement("vehicles");
            v.addElement("weapons");
            v.addElement("computers");
         }
         return v;
      }
      /**********************************************************************/
      public Enumeration getNeeded()
      {
         return needed.keys();
      }
      /**********************************************************************/
      synchronized void updateWealth(String s)
      {
         holdings = (new HoldingsParser(s)).getParsedData();

         updateNeeded();
         updateWillTradeFor();
      }
      /**********************************************************************/
      private synchronized void updateNeeded()
      {
         Hashtable oldNeeded = needed;
         Date now = new Date();
         needed = new Hashtable();
         for (Enumeration e = holdings.keys() ; e.hasMoreElements() ; )
         {
            String holding = (String) e.nextElement();

            Date when = (Date) oldNeeded.get(holding);

            if (getHoldingByName(holding) <= 2)
            {
               if (when == null)
                  needed.put(holding, now);
               else
               {
                  when = new Date(when.getTime()  - 
                              ( (now.getTime() - when.getTime()) / 1000));
                  needed.put(holding, when);
                  System.out.println("DEBUG: Boy, we really need " +
                                     holding +
                                     " so, I upped it to factor of " +
                                     getFactor(holding));
               }
            }
            else
            {
               // This means that this once was a scarce resource
               // Slowly increase our desire to get it
               if (when != null)
               {
                  Date update = new Date(
                                when.getTime()  + 
                                   ( (now.getTime() - when.getTime()) / 1000));
                  if (now.getTime() - update.getTime() <=
                      PlayerParameters.HAVE_ENOUGH_THRESHOLD)
                  {
                     System.out.println("DEBUG: I am not looking for " +
                                        holding + " anymore, I have enough");
                     needed.remove(holding);
                  }
                  else
                  {
                     needed.put(holding, update);
                     System.out.println("DEBUG: Just updated " + holding +
                                      " to a value of " + getFactor(holding) +
                                     " since we have some now.");
                  }
               }
            }
         }
      }
      /**********************************************************************/
      private synchronized void updateWillTradeFor()
      {
         willTradeFor = new Hashtable();

         for (Enumeration e = holdings.keys() ; e.hasMoreElements() ; )
         {
            String thisOne = (String) e.nextElement();
            boolean thisOneIsExpensive = isExpensiveResource(thisOne);

            Vector curVector = new Vector();

            for (Enumeration f = holdings.keys() ; f.hasMoreElements() ; )
            {

               String thatOne = (String) f.nextElement();
               boolean thatOneIsExpensive = isExpensiveResource(thatOne);

               // Don't trade anything expensive for something that's cheap
               if (thisOneIsExpensive && (! thatOneIsExpensive) )
                  continue;
               else if ( (! thisOneIsExpensive) && thatOneIsExpensive )
                  // Always be willing if they offer cheap for expensive
                  curVector.addElement(thatOne);
               else
               {
                  if (getHoldingByName(thisOne) > getHoldingByName(thatOne))
                     curVector.addElement(thatOne);
               }
            }
            willTradeFor.put(thisOne, curVector);
         }
      }
      /**********************************************************************/
      boolean isExpensiveResource(String s)
      {
         return (s.equalsIgnoreCase("computers") ||
                 s.equalsIgnoreCase("weapons") ||
                 s.equalsIgnoreCase("vehicles"));
      }
      /**********************************************************************/
}
/*****************************************************************************/
class PasswordException extends Exception {}
/*****************************************************************************/
class Password implements Serializable {
      public static final String FILE = "./ident.Password";

      static String DH_FILE = "./key.DHKeyObject";

      String identity;
      String playerPassword;
      String playerPasswordChecksum;
      String monitorPassword;
      DHKeyObject dhKey;
      /**********************************************************************/
      Password(String ident, String playPass, String monPas)
      {
         identity = ident;
         playerPassword = playPass;
         monitorPassword = monPas;

         updateChecksum();
         readDH();
         serializePassword();
      }
      /**********************************************************************/
      DHKeyObject getDhKey()
      {
         return dhKey;
      }
      /**********************************************************************/
      void readDH()
      {
         try
         {
            FileInputStream fis = new FileInputStream(DH_FILE);
            ObjectInputStream ois = new ObjectInputStream(fis);

            dhKey = (DHKeyObject) ois.readObject();
         }
         catch (Exception e)
         {
            System.out.println("FATAL: Attempting to read DH file, " +
                               DH_FILE + " : " + e);
         }
      }
      /*********************************************************************/
      String getIdentity()
      {
         return identity;
      }
      /*********************************************************************/
      String getMonitorPassword()
      {
         return monitorPassword;
      }
      /*********************************************************************/
      void setMonitorPassword(String s)
      {
         monitorPassword = s;
         serializePassword();
      }
      /*********************************************************************/
      String getPlayerPassword()
      {
         return playerPassword;
      }
      /*********************************************************************/
      String getPlayerPasswordChecksum()
      {
         return playerPasswordChecksum;
      }
      /**********************************************************************/
      synchronized void changePassword(String playPass, String monPass)
      {
         playerPassword = playPass;
         monitorPassword = monPass;

         updateChecksum();
         serializePassword();
      }
      /**********************************************************************/
      synchronized void serializePassword()
      {
         System.out.println("IDENT  is:           " + identity);
         System.out.println("PLAYER PASSWORD is:  " + playerPassword);
         System.out.println("MONITOR PASSWORD is: " + monitorPassword);

         try
         {
            FileOutputStream fos = new FileOutputStream(FILE);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeObject( (Object) this);
            oos.close();
         }
         catch (Exception e)
         {
            System.out.println("FATAL ERROR -- cannot serialize Password: "
               + e);
         }
      }
      /**********************************************************************/
      void updateChecksum()
      {
         MessageDigest md;
         byte target[]; 

         try
         {
            md = MessageDigest.getInstance("SHA");
         }
         catch (NoSuchAlgorithmException e)
         {
            playerPasswordChecksum = "";
            return;
         }

         target = playerPassword.toUpperCase().getBytes(); 
         md.update(target);
	
         playerPasswordChecksum = 
            (new BigInteger(md.digest()).abs().toString(16));
      }
      /**********************************************************************/
}
/*
  Local Variables:
  tab-width: 4
  indent-tabs-mode: nil
  eval: (c-set-style "ellemtel")
  End:
*/
